﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;

using KBA.GUI;
using KBA.Helpers;

using Styx;
using Styx.Common;
using Styx.Common.Helpers;
using Styx.CommonBot;
using Styx.Helpers;
using Styx.Localization;
using Styx.Plugins;
using Styx.TreeSharp;
using Styx.WoWInternals;

namespace KBA
{

    public class KeepBuffActive : HBPlugin
    {
        #region not HB related
        public static string XmlPath = string.Format("{0}\\Plugins\\KBA\\Data\\BuffList.xml", Utilities.AssemblyDirectory);

        public static DataSet _data = new DataSet("BuffList");

        #endregion
        #region HB related stuff
        public override string Name { get { return "Keep Buffs Active"; } }

        public override string Author { get { return "Stormchasing"; } }

        public override Version Version { get { return new Version(1, 0, 0); } }

        public override bool WantButton { get { return true; } }

        public override string ButtonText { get { return "Setup your Buffs"; } }

        // root of our plugins behavior tree, we need it public cause we wanna stop/start on change of _data
        public static Composite Root;
        // timer used in our behavior tree
        private static WaitTimer waitNextMessage = WaitTimer.FiveSeconds;

        public KeepBuffActive()
        { 
        }

        /// <summary>
        /// Initialize() equals Enabled
        /// ---
        /// called when user clicks the enable checkbox for the plugin, -or when
        /// HonorBuddy loads plugins initially if already initialized
        /// </summary>

        public override void OnEnable()
        {
            base.OnEnable();
            Logger.InfoLog("Enabled");
            // create behavior tree and Start (prepares to run when pulsed)
            Root = CreateBehaviorLogic();
            Root.Start(null);

            // hook events
            BotEvents.OnBotStarted += OnBotStartHandler;
            BotEvents.OnBotStopped += OnBotStopHandler;
        }

        /// <summary>
        /// called when user unchecks your plugin -or- when HonorBuddy is closing.  Any 
        /// resource, hook, or event you implemented in Initialize() must be released here 
        /// to avoid the side-effects of your plugin continuing to run after the user disables
        /// </summary>
        public override void OnDisable()
        {
            Logger.InfoLog("Disabled");

            // unhook events
            BotEvents.OnBotStarted -= OnBotStartHandler;
            BotEvents.OnBotStopped -= OnBotStopHandler;

            // stop the behavior tree and release
            Root.Stop(null);
            Root = null;
            base.OnDisable();
        }


        /// <summary>
        /// Plugin Logic Goes Here
        /// --
        /// This creates the Behavior Tree which represents your plugins logic.
        /// </summary>
        private static Composite CreateBehaviorLogic()
        {
            PrioritySelector p = new PrioritySelector();
            try
            {
                if (!File.Exists(XmlPath))
                {
                    //create default table
                    DataTable table = new DataTable();
                    table.TableName = "BuffList";
                    table.Columns.Add("Description", typeof(string));
                    table.Columns.Add("ItemName", typeof(string));
                    table.Columns.Add("ItemId", typeof(int));
                    table.Columns.Add("BuffName", typeof(string));
                    table.Columns.Add("BuffId", typeof(int));
                    table.Columns.Add("active", typeof(bool));
                    //create default data
                    table.Rows.Add("Potion of Luck", "Potion of Luck", 93351, "Potion of Luck", 135855, false);
                    _data.Tables.Add(table);
                    Logger.InfoLog("Created default data");
                }
                else
                {
                    //load user xml
                    Logger.InfoLog("Loading data from {0}", XmlPath);
                    _data.ReadXml(XmlPath);
                    Logger.InfoLog("Loaded data from xml-file");
                }
                Logger.InfoLog("Creating Plugin-Logic");                
                DataRow[] result = _data.Tables["BuffList"].Select("active=true");
                Logger.InfoLog("Found {0} entries", result.Count());
                foreach (DataRow row in result)
                {
                    object Description = row["Description"];
                    object ItemName = row["ItemName"];
                    object BuffName = row["BuffName"];
                    object ItemId = row["ItemId"];
                    object BuffId = row["BuffId"];
                    //add childs here
                    int _iId = 0;
                    int _bId = 0;
                    int.TryParse(ItemId.ToString(), out _iId);
                    int.TryParse(BuffId.ToString(), out _bId);
                    string _item = (string)ItemName;
                    string _buff = (string)BuffName;
                    Logger.InfoLog("Trying to add KBA-Item {0} - {1} - {2} - {3}", Description, ItemName, ItemId, BuffName, BuffId);
                    //Add childs for Spellids
                    p.AddChild(Spell.Cast(_iId, ret => _iId > 0 && _bId > 0 && !StyxWoW.Me.HasAura(_bId)));
                    p.AddChild(Spell.Cast(_iId, ret => _iId > 0 && _buff.Length > 0 && !StyxWoW.Me.HasAura(_buff)));
                    //Add childs for SpellNames
                    p.AddChild(Spell.Cast(_item, ret => _bId > 0 && !StyxWoW.Me.HasAura(_bId)));
                    p.AddChild(Spell.Cast(_item, ret => _item.Length > 0 && _buff.Length > 0 && !StyxWoW.Me.HasAura(_buff)));
                    //Add childs for bag itemIds
                    p.AddChild(Item.UseBagItem(_iId, ret => _iId > 0 && _bId > 0 && !StyxWoW.Me.HasAura(_bId), (string)Description));
                    p.AddChild(Item.UseBagItem(_iId, ret => _iId > 0 && _buff.Length > 0 && !StyxWoW.Me.HasAura(_buff), (string)Description));
                    //Add childs for bag itemNames
                    p.AddChild(Item.UseBagItem(_item, ret => _item.Length > 0 && _bId > 0 && !StyxWoW.Me.HasAura(_bId), (string)Description));
                    p.AddChild(Item.UseBagItem(_item, ret => _item.Length > 0 && _buff.Length > 0 && !StyxWoW.Me.HasAura(_buff), (string)Description));
                }
            }
            catch (Exception ex)
            {
                Logger.DebugLog(ex.ToString());
            }
            return p;
        }

        /// <summary>
        /// Pulse() equals Behavior Tree Tick()
        /// ---
        /// This will Tick your behavior tree, so does not need to be modified.
        /// Implement your plugins logic in the CreateBehaviorLogic() method.
        /// ---
        /// This style of behavior tree plugin will cause your behavior to tick 
        /// Pulse.  An alternative is to hook one of the predefined TreeHooks
        /// so your tree runs only during the associated state (Rest, Combat, etc.)
        /// For an example of trying to a predefined TreeHook see:
        /// http://www.thebuddyforum.com/honorbuddy-forum/community-developer-forum/114540-answers-treesharp-questions.html#post1114517
        /// </summary>
        public override void Pulse()
        {
            try
            {
                Root.Tick(null);
                // If the last status wasn't running, stop the tree, and restart it.
                if (Root.LastStatus != RunStatus.Running)
                {
                    Root.Stop(null);
                    Root.Start(null);
                }
            }
            catch (Exception e)
            {
                // Restart on any exception.
                Logger.DebugLog(e.ToString());
                Root.Stop(null);
                Root.Start(null);
                throw;
            }
        }
        public override void OnButtonPress()
        {
            var config = new ConfigForm();
            config.ShowDialog();
        }
        /// <summary>
        /// sample Bot Start event handler
        /// </summary>
        public void OnBotStartHandler(EventArgs args)
        {
            DataSet _d = new DataSet("BuffList");
            if (!File.Exists(KeepBuffActive.XmlPath))
            {
                //create default table
                DataTable table = new DataTable();
                table.TableName = "BuffList";
                table.Columns.Add("Description", typeof(string));
                table.Columns.Add("ItemName", typeof(string));
                table.Columns.Add("ItemId", typeof(int));
                table.Columns.Add("BuffName", typeof(string));
                table.Columns.Add("BuffId", typeof(int));
                table.Columns.Add("active", typeof(bool));
                //create default data
                table.Rows.Add("Potion of Luck", "Potion of Luck", 93351, "Potion of Luck", 135855, false);
                _d.Tables.Add(table);
            }
            else
            {
                //load user xml
                _d.ReadXml(KeepBuffActive.XmlPath);
            }
            _data = _d;
            Logger.InfoLog("Bot started, local plugin data initialized.");
        }

        /// <summary>
        /// sample Bot Stop event handler
        /// </summary>
        public void OnBotStopHandler(EventArgs args)
        {
            _data = null;
            Logger.InfoLog("Bot stopped, local plugin data reset.");
        }
    #endregion
    }
}
